﻿using System.Windows;
using System.Threading;
using System.ComponentModel;

namespace BackgroundWorkerInWPF
{
    public partial class WorkerWindow : Window
    {
        private BackgroundWorker backgroundWorker;

        public WorkerWindow()
        {
            InitializeComponent();
            backgroundWorker = 
                ((BackgroundWorker)this.FindResource("backgroundWorker"));
        }

        #region BackgroundWorker Events
        private void BackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            var bgw = sender as BackgroundWorker;
            e.Result = DoSlowProcess((int)e.Argument, bgw, e);
        }

        private void BackgroundWorker_ProgressChanged(object sender, 
            ProgressChangedEventArgs e)
        {
            workerProgress.Value = e.ProgressPercentage;
        }

        private void BackgroundWorker_RunWorkerCompleted(object sender, 
            RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show(e.Error.Message);
            }
            else if (e.Cancelled)
            {
                outputBox.Text = "Canceled";
            }
            else
            {
                outputBox.Text = e.Result.ToString();
                workerProgress.Value = 0;
            }
            startButton.IsEnabled = true;
            cancelButton.IsEnabled = false;
        }
        #endregion

        #region Long-running process
        private int DoSlowProcess(int iterations, 
            BackgroundWorker worker, DoWorkEventArgs e)
        {
            int result = 0;

            for (int i = 0; i <= iterations; i++)
            {
                if (worker != null)
                {
                    if (worker.CancellationPending)
                    {
                        e.Cancel = true;
                        return result;
                    }
                    if (worker.WorkerReportsProgress)
                    {
                        int percentComplete = 
                            (int)((float)i / (float)iterations * 100);
                        worker.ReportProgress(percentComplete);
                    }
                }

                Thread.Sleep(100);
                result = i;
            }

            return result;
        }
        #endregion

        #region UI Events
        private void startButton_Click(object sender, RoutedEventArgs e)
        {
            int iterations = 0;
            if (int.TryParse(inputBox.Text, out iterations))
            {
                backgroundWorker.RunWorkerAsync(iterations);
                startButton.IsEnabled = false;
                cancelButton.IsEnabled = true;
                outputBox.Text = "";
            }
        }

        private void cancelButton_Click(object sender, RoutedEventArgs e)
        {
            this.backgroundWorker.CancelAsync();
        }
        #endregion

    }
}
